# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core.commands import create_command, command_table


def cli_cosmosdb_data_plane_command(name,
                                    operation, client_factory, transform=None,
                                    table_transformer=None, exception_handler=None):
    """Registers an Azure CLI Cosmos DB Data Plane command. These commands always include the
    parameters which can be used to obtain a cosmosdb client."""

    if not exception_handler:
        from ._exception_handler import generic_exception_handler
        exception_handler = generic_exception_handler

    command = create_command(__name__, name, operation, transform, table_transformer,
                             client_factory, exception_handler=exception_handler)

    # add parameters required to create a cosmosdb client
    group_name = 'Cosmos DB Account'

    command.add_argument('db_resource_group_name', '--resource-group-name', '-g',
                         arg_group=group_name,
                         help='name of the resource group. Must be used in conjunction with '
                              'cosmosdb account name.')
    command.add_argument('db_account_name', '--name', '-n', arg_group=group_name,
                         help='Cosmos DB account name. Must be used in conjunction with '
                              'either name of the resource group or cosmosdb account key.')

    command.add_argument('db_account_key', '--key', required=False, default=None,
                         arg_group=group_name,
                         help='Cosmos DB account key. Must be used in conjunction with cosmosdb '
                              'account name or url-connection.')

    command.add_argument('db_url_connection', '--url-connection', required=False, default=None,
                         arg_group=group_name,
                         help='Cosmos DB account url connection. Must be used in conjunction with '
                              'cosmosdb account key.')

    command_table[command.name] = command



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.mgmt.documentdb.models import (
    ConsistencyPolicy,
    DatabaseAccountCreateUpdateParameters,
    Location
)
from azure.mgmt.documentdb.models.document_db_enums import DatabaseAccountKind
from azure.cli.core.util import CLIError
import azure.cli.core.azlogging as azlogging

logger = azlogging.get_az_logger(__name__)

DEFAULT_INDEXING_POLICY = """{
  "indexingMode": "consistent",
  "automatic": true,
  "includedPaths": [
    {
      "path": "/*",
      "indexes": [
        {
          "kind": "Range",
          "dataType": "String",
          "precision": -1
        },
        {
          "kind": "Range",
          "dataType": "Number",
          "precision": -1
        }
      ]
    }
  ]
}"""


def cli_cosmosdb_create(client,
                        resource_group_name,
                        account_name,
                        locations=None,
                        kind=DatabaseAccountKind.global_document_db.value,
                        default_consistency_level=None,
                        max_staleness_prefix=100,
                        max_interval=5,
                        ip_range_filter=None,
                        enable_automatic_failover=None):
    """Create a new Azure Cosmos DB database account."""

    consistency_policy = None
    if default_consistency_level is not None:
        consistency_policy = ConsistencyPolicy(default_consistency_level, max_staleness_prefix,
                                               max_interval)

    from azure.mgmt.resource import ResourceManagementClient
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    resource_client = get_mgmt_service_client(ResourceManagementClient)
    rg = resource_client.resource_groups.get(resource_group_name)
    resource_group_location = rg.location  # pylint: disable=no-member

    if not locations:
        locations.append(Location(location_name=resource_group_location, failover_priority=0))

    params = DatabaseAccountCreateUpdateParameters(
        resource_group_location,
        locations,
        kind=kind,
        consistency_policy=consistency_policy,
        ip_range_filter=ip_range_filter,
        enable_automatic_failover=enable_automatic_failover)

    async_docdb_create = client.create_or_update(resource_group_name, account_name, params)
    docdb_account = async_docdb_create.result()
    docdb_account = client.get(resource_group_name, account_name)  # Workaround
    return docdb_account


def cli_cosmosdb_update(client,
                        resource_group_name,
                        account_name,
                        locations=None,
                        default_consistency_level=None,
                        max_staleness_prefix=None,
                        max_interval=None,
                        ip_range_filter=None,
                        enable_automatic_failover=None):
    """Update an existing Azure Cosmos DB database account. """
    existing = client.get(resource_group_name, account_name)

    update_consistency_policy = False
    if max_interval is not None or \
       max_staleness_prefix is not None or \
       default_consistency_level is not None:
        update_consistency_policy = True

    if max_staleness_prefix is None:
        max_staleness_prefix = existing.consistency_policy.max_staleness_prefix

    if max_interval is None:
        max_interval = existing.consistency_policy.max_interval_in_seconds

    if default_consistency_level is None:
        default_consistency_level = existing.consistency_policy.default_consistency_level

    consistency_policy = None
    if update_consistency_policy:
        consistency_policy = ConsistencyPolicy(default_consistency_level, max_staleness_prefix,
                                               max_interval)
    else:
        consistency_policy = existing.consistency_policy

    if not locations:
        for loc in existing.read_locations:
            locations.append(
                Location(location_name=loc.location_name, failover_priority=loc.failover_priority))

    if ip_range_filter is None:
        ip_range_filter = existing.ip_range_filter

    if enable_automatic_failover is None:
        enable_automatic_failover = existing.enable_automatic_failover

    params = DatabaseAccountCreateUpdateParameters(
        existing.location,
        locations,
        kind=existing.kind,
        consistency_policy=consistency_policy,
        ip_range_filter=ip_range_filter)

    async_docdb_create = client.create_or_update(resource_group_name, account_name, params)
    docdb_account = async_docdb_create.result()
    docdb_account = client.get(resource_group_name, account_name)  # Workaround
    return docdb_account


def cli_cosmosdb_list(client, resource_group_name=None):
    """
    Lists all Azure Cosmos DB database accounts within a given resource group or subscription.
    """
    if resource_group_name:
        return client.list_by_resource_group(resource_group_name)
    else:
        return client.list()


######################
# data plane APIs
######################

# database operations

def _get_database_link(database_id):
    return 'dbs/{}'.format(database_id)


def _get_collection_link(database_id, collection_id):
    return 'dbs/{}/colls/{}'.format(database_id, collection_id)


def _get_offer_link(database_id, offer_id):
    return 'dbs/{}/colls/{}'.format(database_id, offer_id)


def cli_cosmosdb_database_exists(client, database_id):
    """Returns a boolean indicating whether the database exists """
    return len(list(client.QueryDatabases(
        {'query': 'SELECT * FROM root r WHERE r.id=@id',
         'parameters': [{'name': '@id', 'value': database_id}]}))) > 0


def cli_cosmosdb_database_show(client, database_id):
    """Shows an Azure Cosmos DB database """
    return client.ReadDatabase(_get_database_link(database_id))


def cli_cosmosdb_database_list(client):
    """Lists all Azure Cosmos DB databases """
    return list(client.ReadDatabases())


def cli_cosmosdb_database_create(client, database_id):
    """Creates an Azure Cosmos DB database """
    return client.CreateDatabase({'id': database_id})


def cli_cosmosdb_database_delete(client, database_id):
    """Deletes an Azure Cosmos DB database """
    client.DeleteDatabase(_get_database_link(database_id))


# collection operations

def cli_cosmosdb_collection_exists(client, database_id, collection_id):
    """Returns a boolean indicating whether the collection exists """
    return len(list(client.QueryCollections(
        _get_database_link(database_id),
        {'query': 'SELECT * FROM root r WHERE r.id=@id',
         'parameters': [{'name': '@id', 'value': collection_id}]}))) > 0


def cli_cosmosdb_collection_show(client, database_id, collection_id):
    """Shows an Azure Cosmos DB collection and its offer """
    collection = client.ReadCollection(_get_collection_link(database_id, collection_id))
    offer = _find_offer(client, collection['_self'])
    return {'collection': collection, 'offer': offer}


def cli_cosmosdb_collection_list(client, database_id):
    """Lists all Azure Cosmos DB collections """
    return list(client.ReadCollections(_get_database_link(database_id)))


def cli_cosmosdb_collection_delete(client, database_id, collection_id):
    """Deletes an Azure Cosmos DB collection """
    client.DeleteCollection(_get_collection_link(database_id, collection_id))


def _populate_collection_definition(collection,
                                    partition_key_path=None,
                                    indexing_policy=None):
    changed = False

    if partition_key_path:
        if 'partitionKey' not in collection:
            collection['partitionKey'] = {}
        collection['partitionKey'] = {'paths': [partition_key_path]}
        changed = True

    if indexing_policy:
        changed = True
        collection['indexingPolicy'] = indexing_policy
    return changed


def cli_cosmosdb_collection_create(client,
                                   database_id,
                                   collection_id,
                                   throughput=None,
                                   partition_key_path=None,
                                   indexing_policy=DEFAULT_INDEXING_POLICY):
    """Creates an Azure Cosmos DB collection """
    collection = {'id': collection_id}

    options = {}
    if throughput:
        options['offerThroughput'] = throughput

    _populate_collection_definition(collection,
                                    partition_key_path,
                                    indexing_policy)

    created_collection = client.CreateCollection(_get_database_link(database_id), collection,
                                                 options)
    offer = _find_offer(client, created_collection['_self'])
    return {'collection': created_collection, 'offer': offer}


def _find_offer(client, collection_self_link):
    logger.debug('finding offer')
    offers = client.ReadOffers()
    for o in offers:
        if o['resource'] == collection_self_link:
            return o
    return None


def cli_cosmosdb_collection_update(client,
                                   database_id,
                                   collection_id,
                                   throughput=None,
                                   indexing_policy=None):
    """Updates an Azure Cosmos DB collection """
    logger.debug('reading collection')
    collection = client.ReadCollection(_get_collection_link(database_id, collection_id))
    result = {}

    if (_populate_collection_definition(collection,
                                        None,
                                        indexing_policy)):
        logger.debug('replacing collection')
        result['collection'] = client.ReplaceCollection(
            _get_collection_link(database_id, collection_id), collection)

    if throughput:
        logger.debug('updating offer')
        offer = _find_offer(client, collection['_self'])

        if offer is None:
            raise CLIError("Cannot find offer for collection {}".format(collection_id))

        if 'content' not in offer:
            offer['content'] = {}
        offer['content']['offerThroughput'] = throughput

        result['offer'] = client.ReplaceOffer(offer['_self'], offer)
    return result



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import azure.cli.command_modules.cosmosdb._help  # pylint: disable=unused-import


def load_params(_):
    import azure.cli.command_modules.cosmosdb._params  # pylint: disable=redefined-outer-name


def load_commands():
    import azure.cli.command_modules.cosmosdb.commands  # pylint: disable=redefined-outer-name



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=line-too-long

from azure.cli.core.application import APPLICATION
from azure.cli.core.commands import cli_command
from azure.cli.command_modules.cosmosdb._client_factory import (cf_documentdb)
from ._client_factory import get_document_client_factory
from ._command_type import cli_cosmosdb_data_plane_command

mgmt_path = 'azure.mgmt.documentdb.operations.database_accounts_operations#'
custome_path = 'azure.cli.command_modules.cosmosdb.custom#'


def deprecate(argv):
    if argv[0] == 'documentdb':
        from azure.cli.core.util import CLIError
        raise CLIError('All documentdb commands have been renamed to cosmosdb')


APPLICATION.register(APPLICATION.COMMAND_PARSER_PARSING, deprecate)


def db_accounts_factory(_):
    return cf_documentdb().database_accounts


cli_command(__name__, 'cosmosdb show', mgmt_path + 'DatabaseAccountsOperations.get', db_accounts_factory)
cli_command(__name__, 'cosmosdb list-keys', mgmt_path + 'DatabaseAccountsOperations.list_keys', db_accounts_factory)
cli_command(__name__, 'cosmosdb list-read-only-keys', mgmt_path + 'DatabaseAccountsOperations.list_read_only_keys', db_accounts_factory)
cli_command(__name__, 'cosmosdb list-connection-strings', mgmt_path + 'DatabaseAccountsOperations.list_connection_strings', db_accounts_factory)
cli_command(__name__, 'cosmosdb regenerate-key', mgmt_path + 'DatabaseAccountsOperations.regenerate_key', db_accounts_factory)
cli_command(__name__, 'cosmosdb check-name-exists', mgmt_path + 'DatabaseAccountsOperations.check_name_exists', db_accounts_factory)
cli_command(__name__, 'cosmosdb delete', mgmt_path + 'DatabaseAccountsOperations.delete', db_accounts_factory)
cli_command(__name__, 'cosmosdb failover-priority-change', mgmt_path + 'DatabaseAccountsOperations.failover_priority_change', db_accounts_factory)
cli_command(__name__, 'cosmosdb create', custome_path + 'cli_cosmosdb_create', db_accounts_factory)
cli_command(__name__, 'cosmosdb update', custome_path + 'cli_cosmosdb_update', db_accounts_factory)
cli_command(__name__, 'cosmosdb list', custome_path + 'cli_cosmosdb_list', db_accounts_factory)


# # database operations
cli_cosmosdb_data_plane_command('cosmosdb database show', custome_path + 'cli_cosmosdb_database_show', get_document_client_factory)
cli_cosmosdb_data_plane_command('cosmosdb database list', custome_path + 'cli_cosmosdb_database_list', get_document_client_factory)
cli_cosmosdb_data_plane_command('cosmosdb database exists', custome_path + 'cli_cosmosdb_database_exists', get_document_client_factory)
cli_cosmosdb_data_plane_command('cosmosdb database create', custome_path + 'cli_cosmosdb_database_create', get_document_client_factory)
cli_cosmosdb_data_plane_command('cosmosdb database delete', custome_path + 'cli_cosmosdb_database_delete', get_document_client_factory)

# collection operations
cli_cosmosdb_data_plane_command('cosmosdb collection show', custome_path + 'cli_cosmosdb_collection_show', get_document_client_factory)
cli_cosmosdb_data_plane_command('cosmosdb collection list', custome_path + 'cli_cosmosdb_collection_list', get_document_client_factory)
cli_cosmosdb_data_plane_command('cosmosdb collection exists', custome_path + 'cli_cosmosdb_collection_exists', get_document_client_factory)
cli_cosmosdb_data_plane_command('cosmosdb collection create', custome_path + 'cli_cosmosdb_collection_create', get_document_client_factory)
cli_cosmosdb_data_plane_command('cosmosdb collection delete', custome_path + 'cli_cosmosdb_collection_delete', get_document_client_factory)
cli_cosmosdb_data_plane_command('cosmosdb collection update', custome_path + 'cli_cosmosdb_collection_update', get_document_client_factory)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import azure.cli.core.azlogging as azlogging
from azure.cli.core.util import CLIError
from azure.cli.core import __version__ as core_version

logger = azlogging.get_az_logger(__name__)

NO_CREDENTIALS_ERROR_MESSAGE = """
No credentials specified to access Cosmos DB service. Please provide any of the following:
    (1) resource group name and account name
    (2) account name and key
    (3) url-connection and key
"""

UA_AGENT = "AZURECLI/{}".format(core_version)
ENV_ADDITIONAL_USER_AGENT = 'AZURE_HTTP_USER_AGENT'


def _add_headers(client):
    agents = [client.default_headers['User-Agent'], UA_AGENT]
    try:
        agents.append(os.environ[ENV_ADDITIONAL_USER_AGENT])
    except KeyError:
        pass
    client.default_headers['User-Agent'] = ' '.join(agents)


def _get_url_connection(url_collection, account_name):
    if url_collection:
        return url_collection
    elif account_name:
        return 'https://{}.documents.azure.com:443'.format(account_name)
    else:
        return None


def get_document_client_factory(kwargs):
    from pydocumentdb import document_client
    from azure.cli.core.commands.client_factory import get_data_service_client
    from azure.cli.core._profile import CLOUD
    service_type = document_client.DocumentClient

    logger.debug('Getting data service client service_type=%s', service_type.__name__)
    try:
        name = kwargs.pop('db_account_name', None)
        key = kwargs.pop('db_account_key', None)
        url_connection = kwargs.pop('db_url_connection', None)
        resource_group = kwargs.pop('db_resource_group_name', None)

        if name and resource_group and not key:
            # if resource group name is provided find key
            keys = cf_documentdb().database_accounts.list_keys(resource_group, name)
            key = keys.primary_master_key

        url_connection = _get_url_connection(url_connection, name)

        if not key and not url_connection:
            raise CLIError(NO_CREDENTIALS_ERROR_MESSAGE)
        auth = {'masterKey': key}
        client = document_client.DocumentClient(url_connection=url_connection, auth=auth)
    except Exception as ex:
        if isinstance(ex, CLIError):
            raise ex
        # pylint:disable=line-too-long
        raise CLIError(
            'Failed to instantiate an Azure Cosmos DB client using the provided credential ' + str(
                ex))
    _add_headers(client)
    return client


def cf_documentdb(**_):
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    from azure.mgmt.documentdb import DocumentDB
    return get_mgmt_service_client(DocumentDB)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core.help_files import helps  # pylint: disable=unused-import


helps['cosmosdb'] = """
    type: group
    short-summary: Manage Azure Cosmos DB database accounts.
"""

helps['cosmosdb database'] = """
    type: group
    short-summary: Manage Azure Cosmos DB databases.
"""

helps['cosmosdb collection'] = """
    type: group
    short-summary: Manage Azure Cosmos DB collections.
"""

helps['cosmosdb check-name-exists'] = """
    type: command
    short-summary: Checks that the Azure Cosmos DB account name already exists.
"""

helps['cosmosdb create'] = """
    type: command
    short-summary: Create a new Azure Cosmos DB database account.
"""

helps['cosmosdb delete'] = """
    type: command
    short-summary: Deletes an existing Azure Cosmos DB database account.
"""

helps['cosmosdb failover-priority-change'] = """
    type: command
    short-summary: Changes the failover priority for the Azure Cosmos DB database account.
"""

helps['cosmosdb list'] = """
    type: command
    short-summary: Lists all Azure Cosmos DB database accounts within a given resource group or subscription.
"""

helps['cosmosdb list-connection-strings'] = """
    type: command
    short-summary: Lists the connection strings for the specified Azure Cosmos DB database account.
"""

helps['cosmosdb list-keys'] = """
    type: command
    short-summary: Lists the access keys for the specified Azure Cosmos DB database account.
"""

helps['cosmosdb list-read-only-keys'] = """
    type: command
    short-summary: Lists the read-only access keys for the specified Azure Cosmos DB database account.
"""

helps['cosmosdb regenerate-key'] = """
    type: command
    short-summary: Regenerates an access key for the specified Azure Cosmos DB database account.
"""

helps['cosmosdb show'] = """
    type: command
    short-summary: Retrieves the properties of an existing Azure Cosmos DB database account.
"""

helps['cosmosdb update'] = """
    type: command
    short-summary: Update an existing Azure Cosmos DB database account.
"""



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=line-too-long
# pylint: disable=unused-import
from argcomplete.completers import FilesCompleter
from azure.cli.core.commands.parameters import (
    get_resource_name_completion_list,
    enum_choice_list,
    name_type,
    ignore_type,
    file_type)

from azure.cli.core.util import shell_safe_json_parse

from ._client_factory import get_document_client_factory
import azure.cli.core.commands.arm
from azure.mgmt.documentdb.models.document_db_enums import KeyKind
from azure.mgmt.documentdb.models.document_db_enums import DefaultConsistencyLevel
from azure.mgmt.documentdb.models.document_db_enums import DatabaseAccountKind
from azure.mgmt.documentdb.models.failover_policy import FailoverPolicy
from azure.mgmt.documentdb.models.location import Location
from azure.cli.core.commands import register_cli_argument, register_extra_cli_argument, CliArgumentType


def validate_failover_policies(ns):
    """ Extracts multiple space-separated failoverPolicies in regionName=failoverPriority format """
    fp_dict = []
    for item in ns.failover_policies:
        comps = item.split('=', 1)
        fp_dict.append(FailoverPolicy(comps[0], int(comps[1])))
    ns.failover_policies = fp_dict


def validate_locations(ns):
    """ Extracts multiple space-separated locations in regionName=failoverPriority format """
    if ns.locations is None:
        ns.locations = []
        return
    loc_dict = []
    for item in ns.locations:
        comps = item.split('=', 1)
        loc_dict.append(Location(location_name=comps[0], failover_priority=int(comps[1])))
    ns.locations = loc_dict


def validate_ip_range_filter(ns):
    if ns.ip_range_filter:
        ns.ip_range_filter = ",".join(ns.ip_range_filter)


register_cli_argument('cosmosdb', 'account_name', arg_type=name_type, help='Name of the Cosmos DB database account', completer=get_resource_name_completion_list('Microsoft.DocumentDb/databaseAccounts'), id_part="name")

register_cli_argument('cosmosdb regenerate-key', 'key_kind', **enum_choice_list(KeyKind))
register_cli_argument('cosmosdb failover-priority-change', 'failover_policies', validator=validate_failover_policies, help="space separated failover policies in 'regionName=failoverPriority' format. E.g \"East US\"=0 \"West US\"=1", nargs='+')
register_cli_argument('cosmosdb create', 'account_name', completer=None)
register_cli_argument('cosmosdb create', 'resource_group_name', help="name of the resource group")
register_cli_argument('cosmosdb create', 'resource_group_location', ignore_type)
register_cli_argument('cosmosdb create', 'locations', validator=validate_locations, help="space separated locations in 'regionName=failoverPriority' format. E.g \"East US\"=0 \"West US\"=1. Failover priority values are 0 for write regions and greater than 0 for read regions. A failover priority value must be unique and less than the total number of regions. Default: single region account in the location of the specified resource group.", nargs='+')
register_cli_argument('cosmosdb create', 'default_consistency_level', help="default consistency level of the Cosmos DB database account", **enum_choice_list(DefaultConsistencyLevel))
register_cli_argument('cosmosdb create', 'max_staleness_prefix', help="when used with Bounded Staleness consistency, this value represents the number of stale requests tolerated. Accepted range for this value is 1 - 2,147,483,647", type=int)
register_cli_argument('cosmosdb create', 'max_interval', help="when used with Bounded Staleness consistency, this value represents the time amount of staleness (in seconds) tolerated. Accepted range for this value is 1 - 100", type=int)
register_cli_argument('cosmosdb create', 'ip_range_filter', validator=validate_ip_range_filter, help="firewall support. Specifies the set of IP addresses or IP address ranges in CIDR form to be included as the allowed list of client IPs for a given database account. IP addresses/ranges must be comma separated and must not contain any spaces", nargs='+')
register_cli_argument('cosmosdb create', 'kind', help='The type of Cosmos DB database account to create', **enum_choice_list(DatabaseAccountKind))
register_cli_argument('cosmosdb create', 'enable_automatic_failover', help='Enables automatic failover of the write region in the rare event that the region is unavailable due to an outage. Automatic failover will result in a new write region for the account and is chosen based on the failover priorities configured for the account.', type=bool)

register_cli_argument('cosmosdb update', 'account_name', completer=None)
register_cli_argument('cosmosdb update', 'resource_group_name', help="name of the resource group")
register_cli_argument('cosmosdb update', 'locations', validator=validate_locations, help="space separated locations in 'regionName=failoverPriority' format. E.g \"East US\"=0. Failover priority values are 0 for write regions and greater than 0 for read regions. A failover priority value must be unique and less than the total number of regions.", nargs='+')
register_cli_argument('cosmosdb update', 'default_consistency_level', help="default consistency level of the Cosmos DB database account", **enum_choice_list(DefaultConsistencyLevel))
register_cli_argument('cosmosdb update', 'max_staleness_prefix', help="when used with Bounded Staleness consistency, this value represents the number of stale requests tolerated. Accepted range for this value is 1 - 2,147,483,647", type=int)
register_cli_argument('cosmosdb update', 'max_interval', help="when used with Bounded Staleness consistency, this value represents the time amount of staleness (in seconds) tolerated. Accepted range for this value is 1 - 100", type=int)
register_cli_argument('cosmosdb update', 'ip_range_filter', validator=validate_ip_range_filter, help="firewall support. Specifies the set of IP addresses or IP address ranges in CIDR form to be included as the allowed list of client IPs for a given database account. IP addresses/ranges must be comma separated and must not contain any spaces", nargs='+')
register_cli_argument('cosmosdb update', 'enable_automatic_failover', help='Enables automatic failover of the write region in the rare event that the region is unavailable due to an outage. Automatic failover will result in a new write region for the account and is chosen based on the failover priorities configured for the account.', type=bool)

# database id
database_id = CliArgumentType(options_list=('--db-name', '-d'), help='Database Name')
register_cli_argument('cosmosdb database', 'database_id', database_id)
register_cli_argument('cosmosdb collection', 'database_id', database_id)

register_cli_argument('cosmosdb collection', 'collection_id',
                      options_list=('--collection-name', '-c'), help='Collection Name')

register_cli_argument('cosmosdb collection', 'throughput',
                      options_list=('--throughput'), help='Offer Throughput', type=int)

register_cli_argument('cosmosdb collection', 'partition_key_path',
                      options_list=('--partition-key-path'),
                      help='Partition Key Path, e.g., \'/properties/name\'')

register_cli_argument('cosmosdb collection', 'indexing_policy',
                      options_list=('--indexing-policy'),
                      help='Indexing Policy, you can enter it as a string or as a file, e.g., --indexing-policy @policy-file.json)',
                      type=shell_safe_json_parse, completer=FilesCompleter())
